@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999 Bruno Pires Marinho
+ *  Copyright (C) 1999, 2000 Bruno Pires Marinho
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,13 +19,20 @@
 #ifndef _GTM_APPLET_H
 #define _GTM_APPLET_H
 
+/* Applet commands */
+typedef enum {
+    GTM_ACTION_NONE,
+    GTM_ACTION_NEW_URL,
+    GTM_ACTION_OPEN_GTM,
+} GTMAppletCmd;
+
 typedef struct 
 {
     gchar *def_download_dir;
     gboolean addnew_changes_def_dl_dir;
-    gboolean disable_proxy;
-    gboolean disable_auto_dl;
     gboolean confirm_dnd_url;
+    GTMAppletCmd binding_button1_click;
+    gboolean disable_dl_dir;
 } Properties;
 
 /* The applet properties */
