@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999 Bruno Pires Marinho
+ *  Copyright (C) 1999, 2000 Bruno Pires Marinho
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 #include "file-data.h"
 #include "dialogs.h"
 #include "main-window.h"
+#include "download-dirs.h"
+
 
 /* Variables used in the selection of the toolbar style */
 #define NUM_TOOLBAR_TYPES 3
@@ -60,13 +62,16 @@ typedef struct
     GtkWidget *addnew_changes_def_dl_dir;       /* Toggle button */
     GtkWidget *show_list_stat;                  /* Toggle button */
     GtkWidget *show_page_stat;                  /* Toggle button */
-    GtkWidget *confirm_remove;                  /* Toggle button */
+    GtkWidget *confirm_remove_incomplete;       /* Toggle button */
+    GtkWidget *confirm_remove_completed;        /* Toggle button */
     GtkWidget *confirm_restart;                 /* Toggle button */
     GtkWidget *manual_stop_disables_auto_dl;    /* Toggle button */
     GtkWidget *show_dl_info_page;               /* Toggle button */
     GtkWidget *confirm_dnd_url;                 /* Toggle button */
     GtkWidget *use_sound;                       /* Toggle button */
     GtkWidget *use_beep;                        /* Toggle button */
+    GtkWidget *dir_list;                        /* CList */
+    GtkWidget *stop_dl_on_link_loss;            /* Toggle button */
 } PrefDialogData;
 
 
@@ -119,6 +124,13 @@ apply_prefs_auto (PrefDialogData *pref)
             GTK_TOGGLE_BUTTON (pref->manual_stop_disables_auto_dl));
     gnome_config_set_bool ("manual_stop_disables_auto_dl",
                            gtm_pref.manual_stop_disables_auto_dl);
+
+    /* Stop downloads when interface goes down */
+    gtm_pref.stop_dl_on_link_loss =
+        gtk_toggle_button_get_active (
+            GTK_TOGGLE_BUTTON (pref->stop_dl_on_link_loss));
+    gnome_config_set_bool ("stop_dl_on_link_loss",
+                           gtm_pref.stop_dl_on_link_loss);
 }
 
 static void
@@ -157,11 +169,20 @@ apply_prefs_confirmations (PrefDialogData *pref)
     gnome_config_set_bool ("addnew_changes_def_dl_dir",
                            gtm_pref.addnew_changes_def_dl_dir);
 
-    /* Confirm remove */
-    gtm_pref.confirm_remove =
+    /* Confirm remove (not completed) */
+    gtm_pref.confirm_remove_incomplete =
         gtk_toggle_button_get_active (
-            GTK_TOGGLE_BUTTON (pref->confirm_remove));
-    gnome_config_set_bool ("confirm_remove", gtm_pref.confirm_remove);
+            GTK_TOGGLE_BUTTON (pref->confirm_remove_incomplete));
+    gnome_config_set_bool ("confirm_remove_incomplete",
+                           gtm_pref.confirm_remove_incomplete);
+
+
+    /* Confirm remove (completed) */
+    gtm_pref.confirm_remove_completed =
+      gtk_toggle_button_get_active (
+            GTK_TOGGLE_BUTTON (pref->confirm_remove_completed));
+    gnome_config_set_bool ("confirm_remove_completed", 
+                           gtm_pref.confirm_remove_completed);
 
     /* Confirm restart */
     gtm_pref.confirm_restart =
@@ -178,6 +199,11 @@ apply_prefs_confirmations (PrefDialogData *pref)
                            gtm_pref.confirm_dnd_url);
 }
 
+static void
+apply_prefs_directories (PrefDialogData *pref) {
+    download_dirs_apply (pref->dir_list);
+}
+
 static void
 apply_prefs_display (PrefDialogData *pref)
 {
@@ -257,16 +283,19 @@ apply_prefs (GtkWidget *widget, gint page_num, PrefDialogData *pref)
         case 2: /* Download */
             apply_prefs_download (pref);
             break;
+
+        case 3: /* Directories */
+            apply_prefs_directories (pref);
             
-        case 3: /* Confirmations */
+        case 4: /* Confirmations */
             apply_prefs_confirmations (pref);
             break;
             
-        case 4: /* Display */
+        case 5: /* Display */
             apply_prefs_display (pref);
             break;
 
-        case 5: /* Sound */
+        case 6: /* Sound */
             apply_prefs_sound (pref);
             break;
     }    
@@ -449,6 +478,12 @@ create_auto_page (PrefDialogData *pref)
     gtk_box_pack_start (GTK_BOX (vbox), pref->manual_stop_disables_auto_dl,
                         FALSE, FALSE, 0);
 
+    /* Stop auto downloads on link loss */
+    pref->stop_dl_on_link_loss =
+        gtk_check_button_new_with_label (_ ("Stop download on link loss"));
+    gtk_box_pack_start (GTK_BOX (vbox), pref->stop_dl_on_link_loss,
+                        FALSE, FALSE, 0);
+
     /* Set current preferences */
     gtk_entry_set_text(GTK_ENTRY(pref->interface), gtm_pref.interface);
     gtk_spin_button_set_value (GTK_SPIN_BUTTON (pref->chk_interf_time), 
@@ -458,6 +493,9 @@ create_auto_page (PrefDialogData *pref)
     gtk_toggle_button_set_active (
         GTK_TOGGLE_BUTTON (pref->manual_stop_disables_auto_dl),
         gtm_pref.manual_stop_disables_auto_dl);
+    gtk_toggle_button_set_active (
+        GTK_TOGGLE_BUTTON (pref->stop_dl_on_link_loss),
+        gtm_pref.stop_dl_on_link_loss);
 
     /* Connect the signal that acknowledges the change on the widget to 
      * function that turns on the OK and APPLY buttons */
@@ -473,6 +511,9 @@ create_auto_page (PrefDialogData *pref)
     gtk_signal_connect (GTK_OBJECT (pref->manual_stop_disables_auto_dl),
                         "toggled", GTK_SIGNAL_FUNC (pref_modified_cb),
                         (gpointer) pref);
+    gtk_signal_connect (GTK_OBJECT (pref->stop_dl_on_link_loss),
+                        "toggled", GTK_SIGNAL_FUNC (pref_modified_cb),
+                        (gpointer) pref);
 
     return vbox;
 }
@@ -562,6 +603,21 @@ create_download_page (PrefDialogData *pref)
     return vbox;
 }
 
+
+static GtkWidget *
+create_directories_page (PrefDialogData *pref)
+{
+    GtkWidget* widget;
+
+    widget = download_dirs_list_create (&(pref->dir_list));
+    gtk_signal_connect (GTK_OBJECT (pref->dir_list), "grab-focus",
+                        GTK_SIGNAL_FUNC (pref_modified_cb),
+                        (gpointer) pref);
+    
+    return widget;
+}
+
+
 static GtkWidget *
 create_confirmations_page (PrefDialogData *pref)
 {
@@ -578,13 +634,20 @@ create_confirmations_page (PrefDialogData *pref)
     gtk_box_pack_start (GTK_BOX (vbox), pref->addnew_changes_def_dl_dir, FALSE,
                         FALSE, 0);
 
-    /* Confirm remove */
-    pref->confirm_remove = 
+    /* Confirm remove (not completed) */
+    pref->confirm_remove_incomplete = 
         gtk_check_button_new_with_label (_ (
-            "Confirm remove"));
-    gtk_box_pack_start (GTK_BOX (vbox), pref->confirm_remove, FALSE,
+            "Confirm remove when transfer incomplete"));
+    gtk_box_pack_start (GTK_BOX (vbox), pref->confirm_remove_incomplete, FALSE,
                         FALSE, 0);
 
+    /* Confirm remove (completed) */
+    pref->confirm_remove_completed =
+      gtk_check_button_new_with_label (_ (
+            "Confirm remove completed transfers"));
+    gtk_box_pack_start (GTK_BOX (vbox), pref->confirm_remove_completed, FALSE,
+                        FALSE, 0);
+    
     /* Confirm restart */
     pref->confirm_restart = 
         gtk_check_button_new_with_label (_ (
@@ -604,7 +667,9 @@ create_confirmations_page (PrefDialogData *pref)
         pref->addnew_changes_def_dl_dir),
                                   gtm_pref.addnew_changes_def_dl_dir);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
-        pref->confirm_remove), gtm_pref.confirm_remove);
+        pref->confirm_remove_incomplete), gtm_pref.confirm_remove_incomplete);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
+        pref->confirm_remove_completed), gtm_pref.confirm_remove_completed);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
         pref->confirm_restart), gtm_pref.confirm_restart);
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (
@@ -615,7 +680,10 @@ create_confirmations_page (PrefDialogData *pref)
     gtk_signal_connect (GTK_OBJECT (pref->addnew_changes_def_dl_dir),
                         "toggled", GTK_SIGNAL_FUNC (pref_modified_cb),
                         (gpointer) pref);
-    gtk_signal_connect (GTK_OBJECT (pref->confirm_remove), "toggled",
+    gtk_signal_connect (GTK_OBJECT (pref->confirm_remove_incomplete),
+                        "toggled", GTK_SIGNAL_FUNC (pref_modified_cb),
+                        (gpointer) pref);
+    gtk_signal_connect (GTK_OBJECT (pref->confirm_remove_completed), "toggled",
                         GTK_SIGNAL_FUNC (pref_modified_cb),
                         (gpointer) pref);
     gtk_signal_connect (GTK_OBJECT (pref->confirm_restart), "toggled",
@@ -807,6 +875,10 @@ dialog_prefs (void)
     gtk_notebook_append_page (
         GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pref_dialog_data->pbox)->notebook), 
         create_download_page (pref_dialog_data), label);
+    label = gtk_label_new (_ ("Directories"));
+    gtk_notebook_append_page (
+        GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pref_dialog_data->pbox)->notebook), 
+        create_directories_page (pref_dialog_data), label);
     label = gtk_label_new (_ ("Confirmations"));
     gtk_notebook_append_page (
         GTK_NOTEBOOK (GNOME_PROPERTY_BOX (pref_dialog_data->pbox)->notebook), 
