@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999 Bruno Pires Marinho
+ *  Copyright (C) 1999, 2000 Bruno Pires Marinho
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 #ifndef _DIALOGS_H
 #define _DIALOGS_H
 
+#include "file-list.h"
+
 void dialog_about (void);
 
 void dialog_new (void);
@@ -33,4 +35,8 @@ void dialog_prop (gboolean open_dialog, FileData *file_data);
 
 void dialog_add (gchar *uri_list);
 
+void dialog_stats_show (void);
+
+void dialog_stats_update (FileListStats *stats);
+
 #endif /* _DIALOGS_H */
