@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999 Bruno Pires Marinho
+ *  Copyright (C) 1999, 2000 Bruno Pires Marinho
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 #include "gtm.h"
 #include "file-data.h"
 #include "download-info.h"
+#include "dialogs.h"
+#include "main-window.h"
 
 typedef struct
 {
@@ -38,8 +40,8 @@ typedef struct
     GtkWidget *sess_time;        /* Label */
     GtkWidget *sess_speed;       /* Label */
     GtkWidget *local_dir;        /* Label */
-    GtkWidget *disable_proxy;    /* Label */
-    GtkWidget *disable_auto_dl;  /* Label */
+    GtkWidget *disable_proxy;    /* Toggle Button */
+    GtkWidget *disable_auto_dl;  /* Toggle Button */
 } Info;
 
 static Info info;
@@ -224,6 +226,35 @@ create_frame_sess (void)
     return frame;
 }
 
+static void
+toggle_proxy_cb (GtkWidget *widget, gpointer data)
+{
+    FileData *file_data;
+
+    file_data = gtk_clist_get_row_data (GTK_CLIST (dl_file_list),
+                                        GTK_CLIST (dl_file_list)->focus_row);
+    file_data_set_disable_proxy (file_data,
+                                 gtk_toggle_button_get_active (
+                                     GTK_TOGGLE_BUTTON (info.disable_proxy)));
+    download_info_update (file_data);
+    dialog_prop (FALSE, file_data);
+}
+
+static void
+toggle_auto_dl_cb (GtkWidget *widget, gpointer data)
+{
+    FileData *file_data;
+
+    file_data = gtk_clist_get_row_data (GTK_CLIST (dl_file_list),
+                                        GTK_CLIST (dl_file_list)->focus_row);
+    file_data_set_disable_auto_dl (file_data,
+                                   gtk_toggle_button_get_active (
+                                       GTK_TOGGLE_BUTTON (
+                                           info.disable_auto_dl)));
+    download_info_update (file_data);
+    dialog_prop (FALSE, file_data);
+}
+
 static GtkWidget *
 create_frame_options (void)
 {
@@ -244,30 +275,25 @@ create_frame_options (void)
     gtk_container_add (GTK_CONTAINER (frame), sw);
     
     label = gtk_label_new (_ ("Local dir:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                       GTK_SHRINK | GTK_FILL, 0, 0, 0);
     info.local_dir = gtk_label_new ("");
-    gtk_misc_set_alignment (GTK_MISC (info.local_dir), 0.0, 0.5);
+    gtk_misc_set_alignment (GTK_MISC (info.local_dir), 1.0, 0.5);
     gtk_table_attach_defaults (GTK_TABLE (table), info.local_dir, 1, 2, 0, 1);
 
-    label = gtk_label_new (_ ("Disable proxy:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
-                      GTK_SHRINK | GTK_FILL, 0, 0, 0);
-    info.disable_proxy = gtk_label_new ("");
-    gtk_misc_set_alignment (GTK_MISC (info.disable_proxy), 0.0, 0.5);
+    info.disable_proxy = gtk_check_button_new_with_label (_ ("Disable proxy"));
     gtk_table_attach_defaults (GTK_TABLE (table), info.disable_proxy,
-                               1, 2, 1, 2);
+                               0, 2, 1, 2);
+    gtk_signal_connect (GTK_OBJECT (info.disable_proxy), "toggled", 
+                        GTK_SIGNAL_FUNC (toggle_proxy_cb), NULL);
 
-    label = gtk_label_new (_ ("Disable auto download:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
-                      GTK_SHRINK | GTK_FILL, 0, 0, 0);
-    info.disable_auto_dl = gtk_label_new ("");
-    gtk_misc_set_alignment (GTK_MISC (info.disable_auto_dl), 0.0, 0.5);
+    info.disable_auto_dl =
+        gtk_check_button_new_with_label (_ ("Disable auto download"));
     gtk_table_attach_defaults (GTK_TABLE (table), info.disable_auto_dl,
-                               1, 2, 2, 3);
+                               0, 2, 2, 3);
+    gtk_signal_connect (GTK_OBJECT (info.disable_auto_dl), "toggled", 
+                        GTK_SIGNAL_FUNC (toggle_auto_dl_cb), NULL);
 
     return frame;
 }
@@ -329,14 +355,19 @@ download_info_update_total_size (FileData *file_data)
     else if (file_data->total_size == 0)
 	strcpy (buffer, _ ("Unspecified"));
     else
-	sprintf (buffer, "%d Kb", file_data->total_size / 1024);
+	sprintf (buffer, "%d kb", (file_data->total_size + 512) / 1024);
     gtk_label_set_text (GTK_LABEL (info.size), buffer);
-    if (file_data->state != DL_NOT_STARTED)
-        gtk_progress_configure (GTK_PROGRESS (info.retr_progress),
-                                file_data->cur_size, 0, file_data->total_size);
-    else
+
+    if (file_data->state != DL_NOT_STARTED) {
+        if (file_data->cur_size <= file_data->total_size)
+            gtk_progress_configure (GTK_PROGRESS (info.retr_progress),
+                                    file_data->cur_size, 0, 
+                                    file_data->total_size);
+    }
+    else {
         gtk_progress_configure (GTK_PROGRESS (info.retr_progress),
                                 0, 0, file_data->total_size);
+    }
 }
 
 void
@@ -350,7 +381,7 @@ download_info_update_statistics (FileData *file_data)
     if (file_data->state == DL_NOT_STARTED || file_data->state == DL_COMPLETED)
         strcpy (buffer, "");
     else
-        sprintf (buffer, "%d Kb", file_data->cur_size / 1024);
+        sprintf (buffer, "%d kb", (file_data->cur_size + 512) / 1024);
     gtk_label_set_text (GTK_LABEL (info.retrieved), buffer);
     if (file_data->state == DL_COMPLETED)
         gtk_progress_set_percentage (GTK_PROGRESS (info.retr_progress), 1.0);
@@ -418,13 +449,13 @@ download_info_update_statistics (FileData *file_data)
     if (file_data->total_time != 0) {
         if (file_data->state == DL_COMPLETED) {
             if (file_data->total_size != 0)
-                sprintf (buffer, "%.2f Kb/s", 
+                sprintf (buffer, "%.2f kb/s", 
                          ((gfloat) file_data->total_size
                                    / file_data->total_time) / 1024);
             else
                 strcpy (buffer, "");
         } else if (file_data->cur_size != 0)
-            sprintf (buffer, "%.2f Kb/s", 
+            sprintf (buffer, "%.2f kb/s", 
                      ((gfloat) file_data->cur_size / file_data->total_time)
                               / 1024);
         else
@@ -439,7 +470,7 @@ download_info_update_statistics (FileData *file_data)
             if (retr_size == 0)
                 strcpy (buffer, _ ("staled"));
             else
-                sprintf (buffer, "%.2f Kb/s",
+                sprintf (buffer, "%.2f kb/s",
                          ((gfloat) retr_size / file_data->session_elapsed) 
                           / 1024);
         }
@@ -469,8 +500,8 @@ download_info_update (FileData *file_data)
 	gtk_label_set_text (GTK_LABEL (info.sess_time), "");
 	gtk_label_set_text (GTK_LABEL (info.sess_speed), "");
 	gtk_label_set_text (GTK_LABEL (info.local_dir), "");
-	gtk_label_set_text (GTK_LABEL (info.disable_auto_dl), "");
-	gtk_label_set_text (GTK_LABEL (info.disable_proxy), "");
+        gtk_widget_set_sensitive (GTK_WIDGET (info.disable_auto_dl), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET (info.disable_proxy), FALSE);
     } else {
         /* Update state, total size and statistics */
         download_info_update_state (file_data);
@@ -483,13 +514,18 @@ download_info_update (FileData *file_data)
 
 	/* Update options */
 	gtk_label_set_text (GTK_LABEL (info.local_dir), file_data->dir);
-	if (file_data->disable_proxy == 0)
-	    gtk_label_set_text (GTK_LABEL (info.disable_proxy), _ ("no"));
-	else
-	    gtk_label_set_text (GTK_LABEL (info.disable_proxy), _ ("yes"));
-	if (file_data->disable_auto_dl == 0)
-	    gtk_label_set_text (GTK_LABEL (info.disable_auto_dl), _ ("no"));
-	else
-	    gtk_label_set_text (GTK_LABEL (info.disable_auto_dl), _ ("yes"));
+        gtk_widget_set_sensitive (GTK_WIDGET (info.disable_auto_dl), TRUE);
+        gtk_widget_set_sensitive (GTK_WIDGET (info.disable_proxy), TRUE);
+        gtk_toggle_button_set_active 
+            (GTK_TOGGLE_BUTTON (info.disable_auto_dl),
+             file_data->disable_auto_dl);
+        gtk_toggle_button_set_active 
+            (GTK_TOGGLE_BUTTON (info.disable_proxy),
+             file_data->disable_proxy);
+
+        /* Update the state of the toggle auto download button on the toolbar
+           and menu */
+        if (file_data->file_list)
+            menu_toolbar_set_auto_dl (GTK_CLIST (file_data->file_list));
     }
 }
