@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999 Bruno Pires Marinho
+ *  Copyright (C) 1999, 2000 Bruno Pires Marinho
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "download-info.h"
 #include "main-window.h"
 #include "wget-log.h"
+#include "download-dirs.h"
 
 /* Function that returns the state name */
 gchar *
@@ -62,26 +63,36 @@ file_data_state_name (FileData *file_data)
 void
 file_data_free (FileData *file_data)
 {
-    FileData *aux_file_data;
     gchar *path, **files;
-    gint i;
+    gint i, total_files, found;
 
     if (file_data != NULL) {
-        if (file_data->file_list != NULL) {
-            files = g_new0 (gchar *, GTK_CLIST (file_data->file_list)->rows);
-            for (i = 0; i < GTK_CLIST (file_data->file_list)->rows; i++) {
-                aux_file_data =
-                    gtk_clist_get_row_data (GTK_CLIST (file_data->file_list),
-                                            i);
-                files[i] = g_strdup_printf ("%d", aux_file_data->id);
+        gnome_config_get_vector ("/gtm/FileList/file_ids", &total_files,
+                                 &files);
+        if (total_files > 1) {
+            i = 0;
+            found = 0;
+            while (i < total_files - 1) {
+                if (!found && atoi (files[i]) == file_data->id) {
+                    found = 1;
+                    g_free (files[i]);
+                }
+                if (found) {
+                    files[i] = files[i+1];
+                    files[i+1] = NULL;
+                }
+                i++;
             }
             gnome_config_set_vector ("/gtm/FileList/file_ids",
-                                     GTK_CLIST (file_data->file_list)->rows,
+                                     total_files - 1,
                                      (const gchar **) files);
-            for (i = 0; i < GTK_CLIST (file_data->file_list)->rows; i++)
-                g_free (files[i]);
-            g_free (files);
+        } else {
+            next_file_id = 0;
+            gnome_config_clean_section ("/gtm/FileList/");
         }
+        for (i = 0; i < total_files; i++)
+            g_free (files[i]);
+        g_free (files);
         path = g_strdup_printf ("/gtm/File%d/", file_data->id);
         gnome_config_clean_section (path);
         gnome_config_sync ();
@@ -102,6 +113,9 @@ file_data_set_state (FileData *file_data, DlState state)
     gchar *path;
 
     file_data->state = state;
+
+    file_list_update_statistics (GTK_CLIST (file_data->file_list));
+
     if (state == DL_NOT_STARTED || state == DL_NOT_RUNNING
         || state == DL_COMPLETED) {
         path = g_strdup_printf ("/gtm/File%d/state", file_data->id);
@@ -160,6 +174,12 @@ void file_data_set_url (FileData *file_data, gchar *url)
     g_free (path);
 }
 
+int file_data_url_valid (const gchar *url)
+{
+    return (strncasecmp (url, "http://", 7) == 0 ||
+            strncasecmp (url, "ftp://", 6) == 0 );
+}
+
 void file_data_set_disable_proxy (FileData *file_data,
                                   gboolean disable_proxy)
 {
@@ -228,7 +248,9 @@ file_data_update_statistics (FileData *file_data)
             && g_list_find (GTK_CLIST (file_data->file_list)->selection,
                             (gpointer) row) != NULL)
             download_info_update_statistics (file_data);
+        file_list_update_statistics (GTK_CLIST (file_data->file_list));
     }
+    
 }
 
 static gint
@@ -307,7 +329,8 @@ file_data_start_download (FileData *file_data)
             argv[5] = "-c";                   /* Continue download */
             argv[6] = "-t";                   /* Number of retries */
             argv[7] = g_strdup_printf ("%d", gtm_pref.num_retries);
-            arg = 8;
+            argv[8] = "-T 30";                /* Wget timeout */
+            arg = 9;
 
             /* Proxy arguments */
             argv[arg++] = "-Y";
@@ -417,14 +440,14 @@ file_data_stop_download (FileData *file_data)
             if (num_of_download > 0)
                 num_of_download--;
 
-            /* Check if stop action disables auto download */
-            if (gtm_pref.manual_stop_disables_auto_dl)
-                file_data->disable_auto_dl = TRUE;
-
             /* All done this download can be started again */
             file_data->log_tag = -1;
         }
     }
+
+    /* Check if stop action disables auto download */
+    if (gtm_pref.manual_stop_disables_auto_dl)
+        file_data->disable_auto_dl = TRUE;
 }
 
 void
@@ -454,15 +477,18 @@ file_data_create (gchar *url, gchar *dir, gboolean disable_proxy,
     file_data->log_tag = -1;
     file_data->url = g_strdup (url);
 
-    /* If the download dir is empty use the default and if it is not null
-     * make sure there is a '/' in the end of dir name */
+    /* Figure out a directory to use if none given */
     length = strlen (dir);
-    if (length == 0)
-        file_data->dir = g_strdup (gtm_pref.def_download_dir);
-    else if (dir[length - 1] != '/')
-        file_data->dir = g_strconcat (dir, "/", NULL);
-    else
+    if (length == 0) {
+        if (!download_dirs_suggest_dir(file_data->url, &dir))
+            dir = gtm_pref.def_download_dir;
+    }
+    /* Add a trailing '/' unless already present */
+    length = strlen (dir);
+    if (dir[length - 1] == '/')
         file_data->dir = g_strdup (dir);
+    else
+        file_data->dir = g_strconcat (dir, "/", NULL);
 
     /* Get the filename from the URL */
     filename = &file_data->url[strlen (file_data->url)];
@@ -488,7 +514,10 @@ file_data_create (gchar *url, gchar *dir, gboolean disable_proxy,
 
     /* Save the new data */
     file_ids = gnome_config_get_string ("/gtm/FileList/file_ids");
-    new_file_ids = g_strdup_printf ("%s%d ", file_ids, file_data->id);
+    if (file_ids != NULL)
+        new_file_ids = g_strdup_printf ("%s%d ", file_ids, file_data->id);
+    else
+        new_file_ids = g_strdup_printf ("%d ", file_data->id);
     gnome_config_set_string ("/gtm/FileList/file_ids", new_file_ids);
     gnome_config_set_int ("/gtm/FileList/next_file_id", next_file_id);
     prefix = g_strdup_printf ("/gtm/File%d/", file_data->id);
