@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999 Bruno Pires Marinho
+ *  Copyright (C) 1999, 2000 Bruno Pires Marinho
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -56,6 +56,9 @@ typedef struct
     gint drag_motion_y;
 } Panel;
 
+/* list stats */
+static FileListStats *list_stats = NULL;
+
 /* File list popup menu */
 static GnomeUIInfo menu_popup[] = {
 #define MENU_POPUP_REMOVE 0
@@ -183,6 +186,7 @@ file_list_select_row (GtkCList *file_list, gint row, gint column,
     file_data = (FileData *) gtk_clist_get_row_data (file_list,
                                                      file_list->focus_row);
     menu_toolbar_set_state (file_list);
+    menu_toolbar_set_auto_dl (file_list);
     file_list_menu_popup_set_state (file_list);
     download_info_update (file_data);
     dialog_prop (FALSE, file_data);
@@ -193,6 +197,7 @@ file_list_unselect_row (GtkCList *file_list, gint row, gint column,
                         GdkEventButton *event, gpointer data)
 {
     menu_toolbar_set_state (file_list);
+    menu_toolbar_set_auto_dl (file_list);
     file_list_menu_popup_set_state (file_list);
     if (g_list_find (file_list->selection,
                      (gpointer) file_list->focus_row) == NULL) {
@@ -201,6 +206,33 @@ file_list_unselect_row (GtkCList *file_list, gint row, gint column,
     }
 }
 
+/* Save the new list order when the user moves a row. */
+static void
+file_list_row_move (GtkCList *file_list, gint row1, gint row2, gpointer data)
+{
+    gchar *tmp, **files;
+    gint i, total_files;
+
+    gnome_config_get_vector ("/gtm/FileList/file_ids", &total_files, &files);
+
+    /* remove from old position */
+    tmp = files[row1];
+    if (row1 + 1 != total_files)
+        memmove (&files[row1], &files[row1 + 1], 
+                 (total_files - row1 - 1) * sizeof(gchar*));
+    /* insert at new position */
+    memmove (&files[row2+1], &files[row2],
+             (total_files - row2 - 1) * sizeof(gchar*));
+    files[row2] = tmp;
+
+    gnome_config_set_vector ("/gtm/FileList/file_ids", total_files, 
+                             (const gchar**) files);
+    gnome_config_sync();
+
+    for (i = 0; i < total_files; i++)
+        g_free (files[i]);
+}
+
 
 /* Functions used to construct the auto-scroll on drag */
 static GtkAdjustment *
@@ -403,6 +435,9 @@ file_list_create (void)
     gtk_signal_connect (GTK_OBJECT (file_list), "unselect_row", 
                         GTK_SIGNAL_FUNC (file_list_unselect_row),
                         NULL);
+    gtk_signal_connect (GTK_OBJECT (file_list), "row_move",
+                        GTK_SIGNAL_FUNC (file_list_row_move),
+                        NULL);
 
     /* These are for auto-scroll on drag */
     gtk_signal_connect (GTK_OBJECT (file_list), "drag_motion",
@@ -427,6 +462,8 @@ file_list_create (void)
     gtk_signal_connect (GTK_OBJECT (file_list), "drag_data_received",
                         GTK_SIGNAL_FUNC(file_list_drag_data_received), NULL);
 
+    file_list_update_statistics (GTK_CLIST (file_list));
+
     return file_list_sw;
 }
 
@@ -566,27 +603,83 @@ file_list_add (GtkCList *file_list, FileData *file_data)
 }
 
 void 
-file_list_remove_selection (GtkCList *file_list)
+file_list_remove_selection (GtkCList *file_list, gboolean completed,
+                            gboolean incomplete)
 {
     FileData *file_data;
+    GList *p;
     
     gtk_clist_freeze (GTK_CLIST (dl_file_list));
-    while (GTK_CLIST (dl_file_list)->selection != NULL) {
+    p = file_list->selection;
+    while (p != NULL) {
         file_data = 
             (FileData *) gtk_clist_get_row_data (
-                GTK_CLIST (dl_file_list), 
-                (gint) GTK_CLIST (dl_file_list)->selection->data);
-        file_list_remove_file (file_list, file_data);
+                GTK_CLIST (dl_file_list), (gint) p->data);
+        p = p->next;
+        if ((completed && file_data->state == DL_COMPLETED)
+            || (incomplete && file_data->state != DL_COMPLETED))
+            file_list_remove_file (file_list, file_data);
     }
     gtk_clist_thaw (GTK_CLIST (dl_file_list));
 }
 
+static void
+file_list_zero_statistics (FileListStats *stats) 
+{
+    stats->num_stopped = 0;
+    stats->num_running = 0;
+    stats->num_completed = 0;
+    stats->size_stopped = 0;
+    stats->size_running = 0;
+    stats->size_completed = 0;
+    stats->retrieved_stopped = 0;
+    stats->retrieved_running = 0;
+}
+
+void
+file_list_update_statistics (GtkCList *file_list) 
+{
+    gint i;
+    FileData *file_data;
+
+    if (list_stats == NULL)
+        list_stats = g_new (FileListStats, 1);
+    
+    file_list_zero_statistics (list_stats);
+      
+    for (i = 0; i < file_list->rows; i++) {
+        file_data = gtk_clist_get_row_data (file_list, i);
+
+        switch (file_data->state) {
+        case DL_NOT_STARTED: 
+        case DL_NOT_RUNNING:
+            list_stats->num_stopped++;
+            list_stats->size_stopped += file_data->total_size;
+            list_stats->retrieved_stopped += file_data->cur_size;
+            break;
+        case DL_NOT_CONNECTED: 
+        case DL_CONNECTED: 
+        case DL_RETRIEVING:
+            list_stats->num_running++;
+            list_stats->size_running += file_data->total_size;
+            list_stats->retrieved_running += file_data->cur_size; 
+            break;
+        case DL_COMPLETED:
+            list_stats->num_completed++;
+            list_stats->size_completed += file_data->total_size;
+            break;
+        }
+    }
+    dialog_stats_update(list_stats);
+}
+
 void
 file_list_remove_file (GtkCList *file_list, FileData *file_data)
 {
     file_data_stop_download (file_data);
     gtk_clist_remove (file_list, 
                       gtk_clist_find_row_from_data (file_list, file_data));
+    file_list_update_statistics(file_list);
 }
 
 static void
@@ -670,9 +763,10 @@ void
 file_list_load_data (GtkCList *file_list)
 {
     FileData *file_data;
-    gint i, total_files;
+    gint i, j, total_files;
     gchar *prefix, *filename;
     gchar **files;
+    gboolean zero_found;
 
     gtk_clist_freeze (file_list);
     if (gnome_config_has_section ("/gtm/FileList/")) {
@@ -685,42 +779,75 @@ file_list_load_data (GtkCList *file_list)
             /* New list format */
             next_file_id = gnome_config_get_int ("next_file_id=0");
             gnome_config_get_vector ("file_ids", &total_files, &files);
-            gnome_config_pop_prefix ();
+
+            /* Check the file_ids for errors. The errors GTM generated are a
+             * repeated 0 on the list and non existing 0 section */
+            j = 0;
+            zero_found = FALSE;
             for (i = 0; i < total_files; i++) {
-                prefix = g_strdup_printf ("/gtm/File%s/", files[i]);
-                if (gnome_config_has_section (prefix)) {
-                    file_data = g_new (FileData, 1);
-                    gnome_config_push_prefix (prefix);
-                    file_data->id = atoi (files[i]);
-                    file_data->log_tag = -1;
-                    file_data->url = gnome_config_get_string ("url");
-                    file_data->dir = gnome_config_get_string ("dir");
-                    filename = &file_data->url[strlen (file_data->url)];
-                    while (*filename != '/' && filename != file_data->url)
-                        filename--;
-                    filename++;
-                    file_data->filename = g_strdup (filename);
-                    file_data->local_filename =
-                        g_strconcat (file_data->dir,
-                                     file_data->filename,
-                                     NULL);
-                    file_data->disable_proxy = 
-                        gnome_config_get_bool ("disable_proxy=FALSE");
-                    file_data->disable_auto_dl = 
-                        gnome_config_get_bool ("disable_auto_dl=FALSE");
-                    file_data->line = NULL;
-                    file_data->session_start_time = 0;
-                    file_data->session_start_size = 0;
-                    file_data->session_elapsed = 0;
-                    file_data->cur_size = 0;
-                    file_data->state = gnome_config_get_int ("state=0");
-                    file_data->total_size = gnome_config_get_int ("size=0");
-                    file_data->total_time = gnome_config_get_int ("time=0");
-                    file_data->file_list = NULL;
+                if (!strcmp (files[i], "0")) {
+                    if (!zero_found) {
+                        zero_found = TRUE;
+                        if (gnome_config_has_section ("/gtm/File0/"))
+                            files[j++] = files[i];
+                    }
+                } else
+                    files[j++] = files[i];
+            }
+            if (total_files > j) {
+                if (j == 0) {
+                    gnome_config_pop_prefix ();
+                    gnome_config_clean_section ("/gtm/FileList/");
+                    next_file_id = 0;
+                } else {
+                    total_files = j;
+                    gnome_config_set_vector ("file_ids", total_files, 
+                                             (const gchar**) files);
                     gnome_config_pop_prefix ();
-                    file_list_add (file_list, file_data);
                 }
-                g_free (prefix);
+                gnome_config_sync ();
+            }
+
+            /* The list is fixed now we can read it correctly */
+            if (j != 0) {
+                for (i = 0; i < total_files; i++) {
+                    prefix = g_strdup_printf ("/gtm/File%s/", files[i]);
+                    if (gnome_config_has_section (prefix)) {
+                        file_data = g_new (FileData, 1);
+                        gnome_config_push_prefix (prefix);
+                        file_data->id = atoi (files[i]);
+                        file_data->log_tag = -1;
+                        file_data->url = gnome_config_get_string ("url");
+                        file_data->dir = gnome_config_get_string ("dir");
+                        filename = &file_data->url[strlen (file_data->url)];
+                        while (*filename != '/' && filename != file_data->url)
+                            filename--;
+                        filename++;
+                        file_data->filename = g_strdup (filename);
+                        file_data->local_filename =
+                            g_strconcat (file_data->dir,
+                                         file_data->filename,
+                                         NULL);
+                        file_data->disable_proxy = 
+                            gnome_config_get_bool ("disable_proxy=FALSE");
+                        file_data->disable_auto_dl = 
+                            gnome_config_get_bool ("disable_auto_dl=FALSE");
+                        file_data->line = NULL;
+                        file_data->session_start_time = 0;
+                        file_data->session_start_size = 0;
+                        file_data->session_elapsed = 0;
+                        file_data->cur_size = 0;
+                        file_data->state = gnome_config_get_int ("state=0");
+                        file_data->total_size =
+                            gnome_config_get_int ("size=0");
+                        file_data->total_time =
+                            gnome_config_get_int ("time=0");
+                        file_data->file_list = NULL;
+                        gnome_config_pop_prefix ();
+                        file_list_add (file_list, file_data);
+                    }
+                    g_free (prefix);
+                }
             }
             for (i = 0; i < total_files; i++)
                 g_free (files[i]);
@@ -765,6 +892,25 @@ file_list_start_auto_downloads (GtkCList *file_list)
     }
 }
 
+void
+file_list_stop_auto_downloads (GtkCList *file_list)
+{
+    gint row;
+    FileData *file_data;
+    
+    row = 0;
+    while ((file_data = gtk_clist_get_row_data 
+            (GTK_CLIST(file_list), row)) != NULL ) {
+        if (file_data->disable_auto_dl == FALSE && 
+            gtm_pref.stop_dl_on_link_loss) {
+            file_data_stop_download(file_data);
+            /* This is set by file_data_stop_download */
+            file_data->disable_auto_dl = FALSE;
+        }
+        row++;
+    }
+}
+
 void
 file_list_select_all (GtkCList *file_list)
 {
