@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 1999 Bruno Pires Marinho
+ *  Copyright (C) 1999, 2000 Bruno Pires Marinho
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@
 
 void menu_toolbar_set_state (GtkCList *file_list);
 
+void menu_toolbar_set_auto_dl (GtkCList *file_list);
+
 GtkWidget *main_window_create (gchar *geometry);
 
 void toolbar_update_style (void);
@@ -31,6 +33,8 @@ void main_window_update_dl_info_page_visibility (void);
 
 void remove_cmd (GtkWidget *widget, gpointer data);
 
+void stats_cmd (GtkWidget *widget, gpointer data);
+
 void start_cmd (GtkWidget *widget, gpointer data);
 
 void stop_cmd (GtkWidget *widget, gpointer data);
